/* real time linux task (dummy task) for RT-Linux version 0.5  */
/* simulates a 12 bit, 8 channel AD-board                      */
/* in a real world application, the functions kanal0 to kanal7 */
/* must be replaced by functions that read the samples from    */
/* the AD-converter card.                                      */

#include <linux/errno.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <rtl.h>
#include <time.h>
#include <asm/io.h>

#include <pthread.h>
#include <math.h>

#include <rtl_sched.h>
#include <rtl_fifo.h>
#include "control.h"

pthread_t mytask;
int       sr;
char      channels;

/* generates periodic output values         */
/* output range is 12 bit (0 to +4096)      */

int kanal0(int port) {
  static float x=1.0;
  static float dx=0.001;
         int vout=0;
	 
  vout = x*2048.0+2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.001;};
  if (x<=-1.0)   {x=-1; dx=-0.001;};

  return(vout);
}

int kanal1(int port) {
  static float x=1.0;
  static float dx=0.001;
         int vout=0;
	 
  if (dx<=0) vout=-1*x*x*2048.0+4096.0;
  if (dx>0)  vout=   x*x*2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.001;};
  if (x<=-1.0)   {x=-1; dx=-0.001;};

  return(vout);
}

int kanal2(int port) {
  static float x=1.0;
  static float dx=0.001;
         int vout=0;
	 
  vout = x*x*x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.001;};
  if (x<=-1.0)   {x=-1; dx=-0.001;};

  return(vout);
}

int kanal3(int port) {
  static float x=1.0;
  static float dx=0.003;
         int vout=0;
	 
  vout = x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.003;};
  if (x<=-1.0)   {x=-1; dx=-0.003;};

  return(vout);
}

int kanal4(int port) {
  static float x=1.0;
  static float dx=0.003;
         int vout=0;
	 
  if (dx<=0) vout=-1*x*x*2048.0+4096.0;
  if (dx>0)  vout=   x*x*2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.003;};
  if (x<=-1.0)   {x=-1; dx=-0.003;};

  return(vout);
}

int kanal5(int port) {
  static float x=1.0;
  static float dx=0.003;
         int vout=0;
	 
  vout = x*x*x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.003;};
  if (x<=-1.0)   {x=-1; dx=-0.003;};

  return(vout);
}

int kanal6(int port) {
  static float x=1.0;
  static float dx=0.005;
         int vout=0;
	 
  vout = x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.005;};
  if (x<=-1.0)   {x=-1; dx=-0.005;};

  return(vout);
}

int kanal7(int port) {
  static float x=1.0;
  static float dx=0.005;
         int vout=0;
	 
  if (dx<=0) vout=-1*x*x*2048.0+4096.0;
  if (dx>0)  vout=   x*x*2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.005;};
  if (x<=-1.0)   {x=-1; dx=-0.005;};

  return(vout);
}

/* .................................. */
/* function to write samples to fifos */
/* .................................. */
void *fun(void *t) {
  int data, rc;

  int kanal0(int port);
  int kanal1(int port);
  int kanal2(int port);
  int kanal3(int port);
  int kanal4(int port);
  int kanal5(int port);
  int kanal6(int port);
  int kanal7(int port);

  while(1) {
    /* FIFO's 0-7 transfer the samples from channel 1 to 8                       */
    /* FIFO 9 is an indicator whether a FIFO is full when trying to write to it  */
    if (channels==0) pthread_wait_np();
    if ((channels & 1)==1) {
      data=kanal0(0);
      if (rtf_put(0, (char *) &data, 2) < 0) {
	rc=0;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
    if ((channels & 2)==2) {
      data=kanal1(0);
      if (rtf_put(1, (char *) &data, 2) < 0) {
	rc=1;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
    if ((channels & 4)==4) {
      data=kanal2(0);
      if (rtf_put(2, (char *) &data, 2) < 0) {
	rc=2;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
    if ((channels & 8)==8) {
      data=kanal3(0);
      if (rtf_put(3, (char *) &data, 2) < 0) {
	rc=3;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
    if ((channels & 16)==16) {
      data=kanal4(0);
      if (rtf_put(4, (char *) &data, 2) < 0) {
	rc=4;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
    if ((channels & 32)==32) {
      data=kanal5(0);
      if (rtf_put(5, (char *) &data, 2) < 0) {
	rc=5;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
    if ((channels & 64)==64) {
      data=kanal6(0);
      if (rtf_put(6, (char *) &data, 2) < 0) {
	rc=6;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
    if ((channels & 128)==128) {
      data=kanal7(0);
      if (rtf_put(7, (char *) &data, 2) < 0) {
	rc=7;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      pthread_wait_np();
    }
  }
}

int my_handler(unsigned int fifo) {
  struct my_msg_struct msg;
  int err;

  /* FIFO 8 transfers commands to control the driver */
  while ((err=rtf_get(8, &msg, sizeof(msg))) == sizeof(msg)) {
    switch(msg.command) {
    case START_TASK:
      pthread_make_periodic_np(mytask, gethrtime(), HRTICKS_PER_SEC*(float)(1.0/(float)sr));
      pthread_wakeup_np(mytask);
      break;
    case STOP_TASK:
      pthread_suspend_np(mytask);
      break;
    case SET_CH:
      channels=(unsigned char)msg.value;
      break;
    case SET_POL:
      break;
    case SET_SR:
      sr=msg.value;
      break;
    default:
      return -EINVAL;
    }
  }
  if (err!=0) {
    return -EINVAL;
  }
  return 0;
}

int init_module(void)
{
  pthread_attr_t     attr;
  struct sched_param sched_param;

  sr=1;
  channels=0x00;
  rtf_create(0,10000);
  rtf_create(1,10000);
  rtf_create(2,10000);
  rtf_create(3,10000);
  rtf_create(4,10000);
  rtf_create(5,10000);
  rtf_create(6,10000);
  rtf_create(7,10000);
  rtf_create(8,100);  /* input control channel   */
  rtf_create(9,100);  /* fifo overflow indicator */

  pthread_attr_init(&attr);
  sched_param.sched_priority = 2;
  pthread_attr_setschedparam(&attr, &sched_param);
  pthread_create(&mytask,  &attr, fun, (void *)1);
  pthread_setfp_np(mytask,1);
  rtf_create_handler(8, &my_handler);
  return 0;
}

void cleanup_module(void)
{
  rtf_destroy(0);
  rtf_destroy(1);
  rtf_destroy(2);
  rtf_destroy(3);
  rtf_destroy(4);
  rtf_destroy(5);
  rtf_destroy(6);
  rtf_destroy(7);
  rtf_destroy(8);
  rtf_destroy(9);
  pthread_delete_np(mytask);
}
